@@ -1,5 +1,5 @@
 #!/usr/bin/python2.5
-import sys, os, fcntl, gtk, gobject, hildon, dbus, signal, alarm, time
+import sys, os, fcntl, gtk, gobject, hildon, osso, dbus, signal, alarm, time
 import pyconic; conic=pyconic # ideally, import conic, but that's broken
 import syncevolution
 
@@ -134,11 +134,9 @@ class SyncRunner(object):
         os.lseek(fd, 0, 0)
         os.ftruncate(fd, 0)
         break
-    print "lock acquired"
     self.lock_fd = fd
     pid = os.getpid()
     os.write(fd, "%d" % pid)
-    print "locked"
   def release_lock(self):
     os.unlink(self.dir + "/.sync_lock")
     os.close(self.lock_fd)
@@ -160,7 +158,7 @@ class SyncRunner(object):
         proxy_port = self.connection.get_proxy_port(proto)
         proxy = self.method + "://%s:%d" % (proxy_host, proxy_port)
       elif proxy_mode == conic.PROXY_MODE_AUTO:
-        print "Argh, can't handle automatic proxy yet!"
+        print "Can't handle automatic proxy yet!"
         proxy = None
       # all set, launch SyncEvolution
       self.state = self.sync.synchronize_start(self.server, self.config, self.mode, self.sources, proxy)
@@ -804,10 +802,9 @@ class SyncGUI(object):
       self.init_server_config()
 
 ### SYNCHRONIZATION PROGRESS
-  def synchronize_start(self):
+  def synchronize_start(self, mode = None):
     # get parameters
-    mode = None
-    if not self.sync_mode is None:
+    if mode is None and not self.sync_mode is None:
       idx = self.sync_mode.get_active(0)
       mode = default_modes[idx][0]
     sources = None
@@ -851,10 +848,15 @@ class SyncGUI(object):
     elif status["result"] == RESULT_LOCKED:
       banner = hildon.hildon_banner_show_information(self.server_window, "", "Synchronization already in progress")
     elif status["result"] == RESULT_NO_CONNECTION:
-      banner = hildon.hildon_banner_show_information(self.server_window, "", "No network connection")
+      note = hildon.hildon_note_new_information(self.server_window, "Synchronization failed: no network connection")
+      response = gtk.Dialog.run(note)
+    elif status.has_key("status") and status["status"] == 22000:
+      self.synchronize_recover("Select first-time sync mode")
     else:
-#      banner = hildon.hildon_banner_show_information(self.server_window, "", "Synchronization failed")
-      note = hildon.hildon_note_new_information(self.server_window, "Synchronization failed")
+      msg = "Synchronization failed"
+      if status.has_key("error"):
+        msg += ": " + status["error"]
+      note = hildon.hildon_note_new_information(self.server_window, msg)
       response = gtk.Dialog.run(note)
 
   def synchronize_cancel(self, dialog, id):
@@ -883,6 +885,23 @@ class SyncGUI(object):
       self.sync_bar.pulse()
     return True
 
+  def synchronize_recover(self, title):
+    mode_selector = hildon.TouchSelector(text = True)
+    for mode_name, mode_label in default_modes[1:4]:
+      mode_selector.append_text(mode_label)
+    dialog = hildon.PickerDialog(self.server_window)
+    dialog.set_title(title)
+    dialog.set_selector(mode_selector)
+    dialog.show_all()
+    response = dialog.run()
+    mode = None
+    if response == gtk.RESPONSE_OK:
+      idx = mode_selector.get_active(0)
+      mode = default_modes[idx+1][0]
+    dialog.destroy()
+    if not mode is None:
+      self.synchronize_start(mode)
+
 class SyncCLI(object):
   def __init__(self, server, quiet=False):
     gtk.set_application_name("SyncEvolution")
@@ -890,6 +909,8 @@ class SyncCLI(object):
     self.server = server
     self.server_dir = self.sync.get_server_dir(self.server)
     self.server_config = self.sync.get_server_config(self.server)
+    self.osso_ctx = osso.Context("SyncEvolution", "1.0")
+    self.osso_note = osso.SystemNote(self.osso_ctx)
   def synchronize(self):
     self.sync_state = SyncRunner(True, self.sync, self.server, self.server_dir, self.server_config)
     self.sync_state.set_callbacks(self.synchronize_poll)
@@ -901,7 +922,18 @@ class SyncCLI(object):
   def synchronize_cleanup(self):
     status = self.sync_state.finish()
     self.sync_state = None
-    # FIXME: not sure how to show banner without window... so currently not displaying anything...
+    if status["result"] == 0:
+      self.osso_note.system_note_infoprint("Synchronization successful")
+    elif status["result"] == RESULT_LOCKED:
+      # if another synchronization is in progress anyway, this is probably ok to ignore
+      pass
+    elif status["result"] == RESULT_NO_CONNECTION:
+      self.osso_note.system_note_infoprint("Synchronization failed: No connection")
+    else:
+      msg = "Synchronization failed"
+      if status.has_key("error"):
+        msg += ": " + status["error"]
+      self.osso_note.system_note_infoprint(msg)
     if gtk.main_level() > 0:
       gtk.main_quit()
   def synchronize_cancel(self, sig, frame):
